""" Subtract one background image from another one with fish.
VS stands for velocity subtraction which we term as subtracting subsequent frames from one another
so the images become differences between frames instead of the frames themselves.
This can also be known as optic flow or optical flow analysis

NEW USERS will need to customize the data and image paths for where the data is stored, as well as the names of output folders where the images should go

this script will take original images and output velocity-subtracted images organized into image directories in the chosen output folder
the script will also show you outputs of what the resulting subtracted images look like

"""
import cv2
import os
import matplotlib.pyplot as plt
import shutil
import glob, os

import re
from Velocity_subtraction_function import compute_image_difference


source_tuple = ('orpc', 'wells', 'voith')
val_or_test = ('val', 'test')


for valOrTest in val_or_test:
    for fish_src in source_tuple:
        print(f"fish source is {fish_src}, {valOrTest}")
        
        #data that we want to convert
        perc_working_with = "0.3"
        image_directory = "/Users/victoriasabo/Documents/eyesea-yolo/scripts/data/val_and_test_files_dup_rmv/"+fish_src+"/"+valOrTest+"/images" #args.image_directory
        output_img_dir = "/Users/victoriasabo/Documents/eyesea-yolo/scripts/data_preproc_trial/VS_val_and_test0.3_og_plus0_fishless/"+fish_src+"/"+valOrTest+"/images"
        labels_dir = "/Users/victoriasabo/Documents/eyesea-yolo/scripts/data/val_and_test_files_dup_rmv/"+fish_src+"/"+valOrTest+"/labels"
        output_labels_dir = "/Users/victoriasabo/Documents/eyesea-yolo/scripts/data_preproc_trial/VS_val_and_test0.3_og_plus0_fishless/"+fish_src+"/"+valOrTest+"/labels"
        
        #create output folder for images if it doesnt exist
        if not os.path.exists(output_img_dir):
            os.makedirs(output_img_dir)
        else: #this means there are files already there, so we want to remove them
            print(f"removing {len(os.listdir(output_img_dir))} files from {output_img_dir} since it exists")
            for files_ in os.listdir(output_img_dir):
                    file_full_path = os.path.join(output_img_dir, files_)
                    os.remove(file_full_path)

        #create the output folder for labels, if it doesn't exist
        if not os.path.exists(output_labels_dir):
            os.makedirs(output_labels_dir)
        else: #this means there are files already there, so we want to remove them
            print(f"removing {len(os.listdir(output_labels_dir))} files from {output_labels_dir} since it exists")
            for files_ in os.listdir(output_labels_dir):
                    file_full_path = os.path.join(output_labels_dir, files_)
                    os.remove(file_full_path)

        #make sure the images in the input directory are ordered
        fish_img_fpaths = os.listdir(image_directory)
        print(f"found {len(fish_img_fpaths)} fish in the {fish_src}, pre-sorting")
        print(f"the first few image names before sorting are {fish_img_fpaths[0:3]}")
        fish_img_fpaths.sort(key=lambda f: int(re.sub(r'\D', '', f)))
        print(f"the first few image names after sorting are {fish_img_fpaths[0:3]}")
        print(f"found {len(fish_img_fpaths)} fish in the {fish_src}, post-sorting")
        
        #make sure that if the images dont start with same format, that they are not subtracted (i.e., different videos within one folder are not subtracted)
        num_diff_video_streams = 0
        num_fish_imgs_copied_with_no_label = 0
        #now enter a for loop to go through image by image, to create the velocity subtracted image
        for i, img_fpath in enumerate(fish_img_fpaths[:-1]):
            #isolate file name to the left of the _fm_ so we can check if they are the same string of video
            first_img_name = fish_img_fpaths[i]
            second_img_name = fish_img_fpaths[i+1]
            first_img_left_part = first_img_name.split("_fm_")[0]
            second_img_left_part = second_img_name.split("_fm_")[0]
            if first_img_left_part == second_img_left_part:
                #take absolute value of image, and save it as new name of the second image, along with that second image's label
                first_img_read_in = cv2.imread(os.path.join(image_directory, first_img_name))
                second_img_read_in = cv2.imread(os.path.join(image_directory, second_img_name))
                vs_image = compute_image_difference(second_img_read_in, first_img_read_in)
                if i == 1:
                    print(f"showing the velocity subtraction image for {i} iteration")
                    cv2.imshow('image', vs_image)
                    cv2.waitKey(0) 
                    cv2.destroyAllWindows()
                cv2.imwrite(os.path.join(output_img_dir, second_img_name), vs_image) #write the VSdiff image with name of second image
                #now copy the label of second image over to the output label directory
                second_img_assoc_label = second_img_name[:-4] + ".txt"
                second_img_assoc_label_fullpath = os.path.join(labels_dir, second_img_assoc_label)
                if os.path.exists(second_img_assoc_label_fullpath):
                    shutil.copy(second_img_assoc_label_fullpath, os.path.join(output_labels_dir,second_img_assoc_label))
                else:
                    num_fish_imgs_copied_with_no_label += 1

            else:
                print(f"Different video stream started, no VS picture difference image will be created")
                num_diff_video_streams += 1

        print(f"There were {len(os.listdir(output_img_dir))} velocity-subtracted images created from {len(fish_img_fpaths)} images.")
        print(f"There were {num_diff_video_streams} breaks in images, so we expect {len(fish_img_fpaths)-1-num_diff_video_streams} velocity-subtracted images")
        print(f"There were {len(os.listdir(output_labels_dir))} velocity-subtracted labels copied from {len(os.listdir(labels_dir))} labels.")
        print(f"There were {num_fish_imgs_copied_with_no_label} images copied with no label, so if {num_fish_imgs_copied_with_no_label + len(os.listdir(output_labels_dir))} = {len(os.listdir(output_img_dir))}, we're good, otherwise, {len(os.listdir(labels_dir))} - {len(os.listdir(output_labels_dir))} must be equal, plus minus {num_diff_video_streams+1} depending if the first image in any video stream was one that had an associated label")
        
    
        print(f"started with {len(os.listdir(labels_dir))} labels and copied over so we now have {len(os.listdir(output_labels_dir))} labels")
        print(f"started with {len(os.listdir(image_directory))} images, and we subtracted velocity so now there are {len(os.listdir(output_img_dir))} images")

        #lets see the first couple of them:
        for subtracted_img in os.listdir(output_img_dir)[:2]:
            print("example VS image saved in the output image directory is:")
            full_sub_img_path = os.path.join(output_img_dir, subtracted_img)
            subtracted_image = cv2.imread(full_sub_img_path)
            cv2.imshow('image', subtracted_image)
            cv2.waitKey(0)
            cv2.destroyAllWindows()


