""" Create a background image for each source and camera angle
Then subtract it from every photo so theoretically we are left with just the fish

this creates a data set saved to the specified folder, that has 20% fishless images amongst the training set

NEW USER: need to customize the data and folder paths to where things should be saved on your computer
You can also cusotmize the method for the Background subtraction, between MOG2 and KNN

"""
import cv2
import os
import matplotlib.pyplot as plt
import shutil
import glob, os


source_tuple = ('orpc1', 'orpc2','orpc3','orpc4','orpc5',
                'voith1', 'voith2','voith3', 'voith4',
                'wells1', 'wells2')
sub_type = "MOG2" #other option is "KNN" for how you want to figure out the background


for fish_src in source_tuple:
    print(f"fish source is {fish_src}")
    
    #data that we want to convert
    perc_working_with = "0.2"
    image_directory = "/Users/victoriasabo/Documents/eyesea-yolo/Background_Subtraction/data_div_subfolders/"+fish_src+"/fishless_20percent/images" #args.image_directory
    output_img_dir = "/Users/victoriasabo/Documents/eyesea-yolo/Background_Subtraction/data_div_subfolders_BS/"+fish_src+"/fishless_20percent/images"
    labels_dir = "/Users/victoriasabo/Documents/eyesea-yolo/Background_Subtraction/data_div_subfolders/"+fish_src+"/fishless_20percent/labels"
    output_labels_dir = "/Users/victoriasabo/Documents/eyesea-yolo/Background_Subtraction/data_div_subfolders_BS/"+fish_src+"/fishless_20percent/labels"
    
    #data where the fishless images are found
    fishless_image_input_dir = "/Users/victoriasabo/Documents/eyesea-yolo/Background_Subtraction/data_div_subfolders/"+fish_src+"/fishless_20percent/images"
     
    #create output folder for images if it doesnt exist
    if not os.path.exists(output_img_dir):
        os.makedirs(output_img_dir)
    else: #this means there are files already there, so we want to remove them
        print(f"removing {len(os.listdir(output_img_dir))} files from {output_img_dir} since it exists")
        for files_ in os.listdir(output_img_dir):
                file_full_path = os.path.join(output_img_dir, files_)
                os.remove(file_full_path)

    
    #select which type of background subtractor
    if sub_type == "MOG2":
        backSub = cv2.createBackgroundSubtractorMOG2(varThreshold=16, detectShadows=True)
    else:
        backSub = cv2.createBackgroundSubtractorKNN(dist2Threshold=1000, detectShadows=True)


    #get several non fish images with which to create the background
    
    fishless_image_fpaths = os.listdir(fishless_image_input_dir)
    print(f"We will use all fishless images from the list of {len(fishless_image_fpaths)} total fishless images (which is 20% of all from this source).")
    current_fishless_added_images = 0
    
    #since fishless image folder is all fishless images, we can just iterate through
    for fpath in fishless_image_fpaths:
        fpath_img_full_loc = os.path.join(fishless_image_input_dir, fpath)
        #now check if that txt file exists
           
        image = cv2.imread(fpath_img_full_loc)
        # update the background model and obtain foreground mask
        fg_mask = backSub.apply(image) #fg_mask is being overwritten every time, but background is being iterated on and doesnt just write over every time
        current_fishless_added_images += 1
        
        

    #now create the background:
    
    print(f"The background was successfully trained on {current_fishless_added_images} fishless images, and will be saved as its own image before being subtracted from everything in the image directory")
    
    #plot the background with the mask:
    fig, ax = plt.subplots(1, 2, figsize=(15, 7))
    ax[0].imshow(backSub.getBackgroundImage())
    ax[0].set_title(f"Current {sub_type} Background after {current_fishless_added_images} frames {fish_src}")
    ax[1].imshow(fg_mask, cmap='gray')
    ax[1].set_title(f"{sub_type} Foreground Mask after {current_fishless_added_images} frames {fish_src}")
    fig.savefig(f"Background_just_based_on_{fish_src}_{sub_type}_{perc_working_with}_ofallfishlessimgs.png")# Keep track of unique object types to convert to numerical id later

    #save the background object as the official background
    img_background_avg = backSub.getBackgroundImage()
    #save the background to view later:
    output_base_dir = output_img_dir[:-12] #take off the '/fish/images'
    path_for_background_detected = os.path.join(output_base_dir, f"background_detected_using{current_fishless_added_images}fishlessimgs.png") #add in another file folder where this will live
    print(f"the background image is saved at {path_for_background_detected}")
    
    cv2.imwrite(path_for_background_detected, img_background_avg)
    cv2.imshow('image', img_background_avg)
    cv2.waitKey(0) 
    cv2.destroyAllWindows()

    #now go through a loop and subtract out this image from every other image
    # and then save each image as the exact same title but in a new output folder
    #it must maintain the same name otherwise the labels will not work
    fish_image_fpaths = os.listdir(image_directory)
    
    iter = 0
    for sing_img in fish_image_fpaths:
        full_sing_img_path = os.path.join(image_directory, sing_img)
        image = cv2.imread(full_sing_img_path)
        # now subtract the background from this image:
        subtracted_remainder = cv2.subtract(image, img_background_avg)
        if (iter%2000) == 0:
            cv2.imshow('image', subtracted_remainder)
            cv2.waitKey(0) 
            cv2.destroyAllWindows()
            print(f"iteration {iter} for fish source {fish_src}")
        if (iter == 0):
            print(f"iteration 1 has begun on {fish_src}")
        path_for_subtracted_file = os.path.join(output_img_dir, sing_img)
        cv2.imwrite(path_for_subtracted_file, subtracted_remainder)
        iter +=1

    print(f"went through loop {iter} times and should have gone through {len(fish_image_fpaths)} times, fish source {fish_src}")
    #now copy over the label file too

    print(f"started with {len(os.listdir(image_directory))} images, and we subtracted the background from each and saved them so now there are {len(os.listdir(output_img_dir))} images")

    #lets see the first couple of them:
    for subtracted_img in os.listdir(output_img_dir)[:1]:
        full_sub_img_path = os.path.join(output_img_dir, subtracted_img)
        subtracted_image = cv2.imread(full_sub_img_path)
        cv2.imshow('image', subtracted_image)
        cv2.waitKey(0)
        cv2.destroyAllWindows()


