""" This creates data folders of just fishless images that were background subtracted then underwent CLAHE image distinction augmentation
Namely, a background image for each source and camera angle was created, 
Then it was subtracted from every photo so theoretically we are left with just the differences
Then CLAHE was applied to those BS-ed images, to augment the differences even more

this creates a data set saved to the specified folder, that has just fishless images

NEW USER: need to customize the data and folder paths to where output images should be saved on your computer
note, for Clahe, the clip limit and grid size can also be customized, although the ones used here were seen to be most effective
"""
import cv2
import os
import matplotlib.pyplot as plt
import shutil

source_tuple = ('orpc1', 'orpc2','orpc3','orpc4','orpc5',
                'voith1', 'voith2','voith3', 'voith4',
                'wells1', 'wells2')

for fish_src in source_tuple:
    print(f"fish source is {fish_src}")
    set_cliplimit = 2.0
    set_gridsize = (15,15)
    image_directory = "/Users/victoriasabo/Documents/eyesea-yolo/Background_Subtraction/data_div_subfolders_BS/"+fish_src+"/fishless_20percent/images" #args.image_directory
    output_img_dir = "/Users/victoriasabo/Documents/eyesea-yolo/Background_Subtraction/data_div_subfolders_BS_then_CLAHE/"+fish_src+"/fishless_20percent/images"
    labels_dir = "/Users/victoriasabo/Documents/eyesea-yolo/Background_Subtraction/data_div_subfolders_BS/"+fish_src+"/fishless_20percent/labels"
    output_labels_dir = "/Users/victoriasabo/Documents/eyesea-yolo/Background_Subtraction/data_div_subfolders_BS_then_CLAHE/"+fish_src+"/fishless_20percent/labels"
    
    
    #create output folder for images if it doesnt exist
    if not os.path.exists(output_img_dir):
        os.makedirs(output_img_dir)
    else: #this means there are files already there, so we want to remove them
        print(f"removing {len(os.listdir(output_img_dir))} files from {output_img_dir} since it exists")
        for files_ in os.listdir(output_img_dir):
                file_full_path = os.path.join(output_img_dir, files_)
                os.remove(file_full_path)

    
    
    clahe = cv2.createCLAHE(clipLimit=set_cliplimit, tileGridSize=set_gridsize)

    #now create the CLAHEd images:
    image_fpaths = os.listdir(image_directory)
    print(f"getting files from {image_directory}")
    print(f"number of images in input image directory is {len(image_fpaths)}")
    print(f"one example of a file name is {image_fpaths[0]}")
    for i, img_short_name in enumerate(image_fpaths):
        img_path = os.path.join(image_directory, img_short_name)
        image_incolor = cv2.imread(img_path)
        img_lab = cv2.cvtColor(image_incolor, cv2.COLOR_BGR2LAB)
        l, a, b = cv2.split(img_lab)
        clahe_l = clahe.apply(l)
        lab_clahe = cv2.merge((clahe_l, a, b))
        clahe_img = cv2.cvtColor(lab_clahe, cv2.COLOR_LAB2BGR)

        #if there is no image resulting from this, then do not go on to the save text
        if clahe_img is None:
            print(f"entered into the continue loop on iteration {i}")
            continue
        #save CLAHE-ed image to separate file
        output_img_path = os.path.join(output_img_dir, img_short_name)
        cv2.imwrite(output_img_path, clahe_img)

        #output progress and the images so I can see what the effect is
        if i%500 == 0:
            print(f"just went through {i}th iteration and has saved {len(os.listdir(output_img_dir))} images to output image folder as of right now")
            
        #choose arbitrary image to save a side-by-side for, so that the pre-processing can be viewed
        if i == 20:
            #now make a file with both side by side
            fig, ax = plt.subplots(1, 2, figsize=(15, 7))
            ax[0].imshow(clahe_img)
            ax[0].set_title(f"Example {fish_src} image with CLAHE applied, cliplimit {set_cliplimit}, gridsize {set_gridsize[1:3]}")
            ax[1].imshow(image_incolor, cmap='gray')
            ax[1].set_title(f"Example {fish_src} image before CLAHE")
            fig.savefig(f"Exp3.0.1_samp_BS_then_CLAHEfishless_{fish_src}_{i}iter_{set_cliplimit}CL_{set_gridsize[1:3]}GS.png")# Keep track of unique object types to convert to numerical id later


    print(f"went through for loop {i+1} times and should have gone through {len(image_fpaths)} times, fish source {fish_src}")
    #now copy over the label file too

    print(f"started with {len(os.listdir(image_directory))} images, and we have performed histogram euqlaization on each and saved them so now there are {len(os.listdir(output_img_dir))} images")

    #lets see the first couple of them:
    for clahe_img in os.listdir(output_img_dir)[:2]:
        full_clahe_img_path = os.path.join(output_img_dir, clahe_img)
        clahe_image = cv2.imread(full_clahe_img_path)
        cv2.imshow('image', clahe_image)
        cv2.waitKey(0)
        cv2.destroyAllWindows()


