""" Creates dataset that has been background subtracted, but this time,
the background image that is subtracted from every other image is made sure to have been
just fishless images from the same continguous video of fish images, as to not
have objects appear in the BS-ed images that are not fish, but rather differences in background
because each different video, although from the same camera and angle, might have a different element
present in the background that isnt present in every other stream of video from that same angle
(for example, like a rope in the field of vision that is at a different elevation
in each contiguous tream of video,
so creating a new background image for each video will ensure the rope is subtracted out
every time no matter where it is in the frame)

This script creates the training datasets, while the other similar script creates the validation/ testing datasets

NEW USER: need to customize the data and folder paths to where images and files are saved on your computer

"""
import cv2
import os
import matplotlib.pyplot as plt
import shutil
import glob, os


source_tuple = ('Igiugig_1', 'Igiugig_2','Igiugig_3','Voith', 'WellsDam')
sub_type = "MOG2" #other option is "KNN" for how you want to figure out the background


for fish_src in source_tuple:
    print(f"fish source is {fish_src}")
    # define which images we are going to work with, and how many images are going to be used to create the background
    
    perc_working_with = "0.5"
    image_directory = "/Users/victoriasabo/Documents/eyesea-yolo/scripts/data_smaller_percent_total_images/train0.5_og_plus20_fishless/"+fish_src+"/images" #args.image_directory
    output_img_dir = "/Users/victoriasabo/Documents/eyesea-yolo/scripts/data_BS_trial/BS_train0.5_og_plus20_fishless/"+fish_src+"/images"
    labels_dir = "/Users/victoriasabo/Documents/eyesea-yolo/scripts/data_smaller_percent_total_images/train0.5_og_plus20_fishless/"+fish_src+"/labels"
    output_labels_dir = "/Users/victoriasabo/Documents/eyesea-yolo/scripts/data_BS_trial/BS_train0.5_og_plus20_fishless/"+fish_src+"/labels"
    num_images_used_to_form = 20 #len(os.listdir(image_directory)), setting numebr to be number of fishless I know to exist in the datasets


    
    #create output folder for images if it doesnt exist
    if not os.path.exists(output_img_dir):
        os.makedirs(output_img_dir)
    else: #this means there are files already there, so we want to remove them
        print(f"removing {len(os.listdir(output_img_dir))} files from {output_img_dir} since it exists")
        for files_ in os.listdir(output_img_dir):
                file_full_path = os.path.join(output_img_dir, files_)
                os.remove(file_full_path)

    #create the output folder for labels, if it doesn't exist
    if not os.path.exists(output_labels_dir):
        os.makedirs(output_labels_dir)
    else: #this means there are files already there, so we want to remove them
        print(f"removing {len(os.listdir(output_labels_dir))} files from {output_labels_dir} since it exists")
        for files_ in os.listdir(output_labels_dir):
                file_full_path = os.path.join(output_labels_dir, files_)
                os.remove(file_full_path)


    #select which type of background subtractor
    if sub_type == "MOG2":
        backSub = cv2.createBackgroundSubtractorMOG2(varThreshold=16, detectShadows=True)
        # backSub.setShadowThreshold(0.75)
    else:
        backSub = cv2.createBackgroundSubtractorKNN(dist2Threshold=1000, detectShadows=True)


    #get several non fish images with which to create the background
    
    image_fpaths = os.listdir(image_directory)
    print(f"We will add {num_images_used_to_form} fishless images from the list of {len(image_fpaths)} total fish and fishless images.")
    #find all label files
    to_test_fishless_label_fpaths = glob.glob("*.txt", root_dir=labels_dir)
    print(f"Found {len(to_test_fishless_label_fpaths)} fish image labels to help identify which {num_images_used_to_form} of the {len(image_fpaths)} images do not have fish so we can make the background from them.")
    current_fishless_added_images = 0
    iter_find_fishless = 1
    
    for fpath in image_fpaths:
        fpath_label_name = fpath[:-4] + ".txt"
        #make file name for hypothetical txt file that will only exist if the image has a fish in it
        fpath_txt_version = os.path.join(labels_dir, fpath_label_name)
        fpath_img_full_loc = os.path.join(image_directory, fpath)
        #now check if that txt file exists
        if not os.path.exists(fpath_txt_version):
            print(f"found fishless image for {fish_src} on iteration {iter_find_fishless}.")
            image = cv2.imread(fpath_img_full_loc)
            # update the background model and obtain foreground mask
            fg_mask = backSub.apply(image) #fg_mask is being overwritten every time, but background is being iterated on and doesnt just write over every time
            current_fishless_added_images += 1
        if current_fishless_added_images >= int(num_images_used_to_form):
            #check if the number of fishless images added were greater than or equal to number we want to include
            #if so, we stop the for loop
            print(f"we are breaking the training of the background because we've gotten to {current_fishless_added_images} which is more than the {num_images_used_to_form} we want.")
            break
        iter_find_fishless += 1

    #now create the background:
    
    print(f"The background was successfully trained on {current_fishless_added_images} fishless images in {iter_find_fishless} iterations, and will be saved as its own image before being subtracted from everything in the image directory")
    
    #plot the background with the mask:
    fig, ax = plt.subplots(1, 2, figsize=(15, 7))
    ax[0].imshow(backSub.getBackgroundImage())
    ax[0].set_title(f"Current {sub_type} Background after {current_fishless_added_images} frames")
    ax[1].imshow(fg_mask, cmap='gray')
    ax[1].set_title(f"{sub_type} Foreground Mask after {current_fishless_added_images} frames")
    fig.savefig(f"test_output_subtract_imgs_{fish_src}_{sub_type}_{perc_working_with}_backjustnonfish.png")# Keep track of unique object types to convert to numerical id later

    #save the background object as the official background
    img_background_avg = backSub.getBackgroundImage()
    #save the background to view later:
    output_base_dir = output_img_dir[:-7] #take off the '/images'
    path_for_background_detected = os.path.join(output_base_dir, f"background_detected_using{current_fishless_added_images}frames.png") #add in another file folder where this will live
    print(f"the background image is saved at {path_for_background_detected}")
    
    cv2.imwrite(path_for_background_detected, img_background_avg)
    cv2.imshow('image', img_background_avg)
    cv2.waitKey(0) 
    cv2.destroyAllWindows()

    #now go through a loop and subtract out this image from every other image
    # and then save each image as the exact same title but in a new output folder
    #it must maintain the same name otherwise the labels will not work
    iter = 0
    for sing_img in image_fpaths:
        full_sing_img_path = os.path.join(image_directory, sing_img)
        image = cv2.imread(full_sing_img_path)
        # now subtract the background from this image:
        subtracted_remainder = cv2.subtract(image, img_background_avg)
        if (iter%1000) == 0:
            cv2.imshow('image', subtracted_remainder)
            cv2.waitKey(0) 
            cv2.destroyAllWindows()
            print(f"iteration {iter} for fish source {fish_src}")
        if (iter == 0):
            print(f"iteration 1 has begun on {fish_src}")
        path_for_subtracted_file = os.path.join(output_img_dir, sing_img)
        #print(type(subtracted_remainder))
        cv2.imwrite(path_for_subtracted_file, subtracted_remainder)
        iter +=1

    print(f"went through loop {iter} times and should have gone through {len(image_fpaths)} times, fish source {fish_src}")
    
    #now copy over the label file too
    shutil.copytree(labels_dir, output_labels_dir, dirs_exist_ok=True)
    print(f"started with {len(os.listdir(labels_dir))} labels and copied over so we now have {len(os.listdir(output_labels_dir))} labels")
    print(f"started with {len(os.listdir(image_directory))} images, and we subtracted the background from each and saved them so now there are {len(os.listdir(output_img_dir))} images")

    #lets see the first couple of them:
    for subtracted_img in os.listdir(output_img_dir)[:2]:
        full_sub_img_path = os.path.join(output_img_dir, subtracted_img)
        subtracted_image = cv2.imread(full_sub_img_path)
        cv2.imshow('image', subtracted_image)
        cv2.waitKey(0)
        cv2.destroyAllWindows()


