import os
from glob import glob
from natsort import natsorted


def get_files(pattern):
    """Returns a list of relative file paths based on the given pattern.

    Args:
        pattern (str): Search pattern to find the desired files.

    Returns:
        list, list: Sorted lists of relative file paths found by the search pattern.
    """
    image_paths = get_images(pattern)
    label_paths = get_labels(image_paths)
    return image_paths, label_paths


def get_images(pattern):
    """Returns a list of relative file paths based on the given pattern.

    Args:
        pattern (str): Search pattern to find the desired files.

    Returns:
        list: Sorted list of relative file paths found by the search pattern.
    """
    files = glob(pattern)
    return nat_sort(files)


def get_labels(image_paths):
    """Uses the image paths to find the neighboring label paths.

    Args:
        image_paths (list): List of paths to images.

    Returns:
        list: List of paths to corresponding labels.
    """
    label_paths = []
    for image_path in image_paths:
        label_path = image_path.replace("_x", "_y").replace(".png", ".xml")
        assert os.path.exists(label_path), f"Label path {label_path} does not exist."
        label_paths.append(label_path)
    return label_paths


def nat_sort(strings):
    """Sorts a list of strings "naturally".

    Args:
        strings (list): List of strings.

    Returns:
        list: Naturally sorted list of strings.
    """
    return natsorted(strings)


def write_label(path, object_strings):
    """Takes a string that encodes a labeled object and writes it to file.

    Args:
        path (str): Path to write file.
        object_strings (list): List of strings that encodes labeled objects.
    """
    with open(path, "w") as file:
        file.write("\n".join(object_strings))

def find_image_dirs(src):
    image_dirs = set()
    for root, dirs, files in os.walk(src):
        for file in files:
            if file.endswith(".png"):
                image_dirs.add(root)
    return image_dirs